/*
 * CWS_DirectoryContainsFile.c
 *
 * (C) Copyright IBM Corp. 2003, 2005, 2010
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Eclipse Public License from
 * http://www.opensource.org/licenses/eclipse-1.0.php
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>

 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: CMPI Workshop DirectoryContainsFile Association Provider.
 * 
 */


#include "CWS_FileUtils.h"
#include "util/cwsutil.h"
#include <cmpidt.h>
#include <cmpift.h>
#include <cmpimacs.h>
#include <string.h>
#include <libgen.h>
#include <stdio.h>

#define LOCALCLASSNAME "CWS_DirectoryContainsFile"
#define DIRECTORYCLASS "CWS_Directory"
#define FILECLASS      "CWS_PlainFile"

static const CMPIBroker * _broker;

/* ------------------------------------------------------------------ *
 * Simplified Utility Functions
 * no support for assocClass, resultClass, etc.
 * ------------------------------------------------------------------ */

static CMPIStatus returnAssocName(const CMPIResult * rslt,
				  const CMPIObjectPath * cop,
				  char * clsname,
				  CWS_FILE *filebuf,
				  const char * assocClass,
				  const char * resultClass)
{
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  CMPIObjectPath *op;

  char * namespace = CMGetCharPtr(CMGetNameSpace(cop, NULL));  

  /* Check that the requested association class, if any, is supported. */
  if (assocClass != NULL) {
     CMPIObjectPath * assoc = CMNewObjectPath(_broker, namespace, LOCALCLASSNAME, NULL);
     if (!CMClassPathIsA(_broker, assoc, assocClass, NULL)) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_INVALID_CLASS, 
			     "Unrecognized association class. Ignoring request.");
	return st;
     }
  }

  /* build object path from file buffer */
  op = CWS_makePath(_broker, 
		clsname,
		CMGetCharPtr(CMGetNameSpace(cop,NULL)), 
		filebuf);

  if (CMIsNullObject(op)) {
     CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			  "Could not construct object path");
     return st;
  }

  char * class = CMGetCharPtr(CMGetClassName(op,NULL));
  CMPIObjectPath * result = CMNewObjectPath(_broker, namespace, class, NULL);

  if (resultClass == NULL || (resultClass != NULL && 
      CMClassPathIsA(_broker, result, resultClass, NULL))) {
      CMReturnObjectPath(rslt,op);
  }

  return st;
}

static CMPIStatus returnRefName(const CMPIResult * rslt,
				const CMPIObjectPath * cop,
				char * clsname,
				CWS_FILE *filebuf,
				const char * assocClass)
{
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  CMPIObjectPath *op;
  CMPIObjectPath *opRef;

  char * namespace = CMGetCharPtr(CMGetNameSpace(cop, NULL));  

  /* Check that the requested association class, if any, is supported. */
  if (assocClass != NULL) {
     CMPIObjectPath * assoc = CMNewObjectPath(_broker, namespace, LOCALCLASSNAME, NULL);
     if (!CMClassPathIsA(_broker, assoc, assocClass, NULL)) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_INVALID_CLASS, 
			     "Unrecognized association class. Ignoring request.");
	return st;
     }
  }

  /* build object path from file buffer */
  op = CWS_makePath(_broker, 
		clsname,
		CMGetCharPtr(CMGetNameSpace(cop,NULL)), 
		filebuf);
  if (CMIsNullObject(op)) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not construct object path");
    return st;
  }
  /* make reference object path */
  opRef = CMNewObjectPath(_broker,
			  CMGetCharPtr(CMGetNameSpace(cop,NULL)), 
			  LOCALCLASSNAME,
			  NULL);
  if (CMIsNullObject(op)) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not construct object path");
    return st;
  }
  if (strcmp(clsname,FILECLASS)==0) {
    CMAddKey(opRef,"GroupComponent",&cop,CMPI_ref);
    CMAddKey(opRef,"PathComponent",&op,CMPI_ref);
  } else {
    CMAddKey(opRef,"GroupComponent",&op,CMPI_ref);
    CMAddKey(opRef,"PathComponent",&cop,CMPI_ref);
  }
  CMReturnObjectPath(rslt,opRef);
  return st;
}

static CMPIStatus associatorHelper( const CMPIResult * rslt,
				    const CMPIObjectPath * cop,
				    int associators, int names, 
				    const char * assocClass,
				    const char * resultClass) 
{
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  CMPIString     *clsname;
  CMPIData        data;
  void           *enumhdl;
  CWS_FILE        filebuf;

  fprintf(stderr,"--- associatorHelper()\n");
    
  /*
   * Check if the object path belongs to a supported class
   */
  clsname = CMGetClassName(cop,NULL);
  if (clsname) {
    if (CMClassPathIsA(_broker,cop,DIRECTORYCLASS,NULL)) {
      /* we have a directory and can return the children */
      data = CMGetKey(cop,"Name",NULL);
      
      enumhdl = CWS_Begin_Enum(CMGetCharPtr(data.value.string),CWS_TYPE_PLAIN,0);
      
      if (enumhdl == NULL) {
	CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			     "Could not begin file enumeration");
	return st;
      } else {
	while (CWS_Next_Enum(enumhdl,&filebuf)) {
	  if (names) {
	    if (associators) {
	      st = returnAssocName(rslt,cop,FILECLASS,&filebuf,assocClass,resultClass);
	    } else {
	      st = returnRefName(rslt,cop,FILECLASS,&filebuf,assocClass);
	    }
	  }
	  if (st.rc != CMPI_RC_OK) break;
	}
	CWS_End_Enum(enumhdl);
      }

      /* now the directories */
      enumhdl = CWS_Begin_Enum(CMGetCharPtr(data.value.string),CWS_TYPE_DIR,0);
      
      if (enumhdl == NULL) {
	CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			     "Could not begin file enumeration");
	return st;
      } else {
	while (CWS_Next_Enum(enumhdl,&filebuf)) {
	  if (names) {
	    if (associators) {
	      st = returnAssocName(rslt,cop,DIRECTORYCLASS,&filebuf,assocClass,resultClass);
	    } else {
	      st = returnRefName(rslt,cop,DIRECTORYCLASS,&filebuf,assocClass);
	    }
	  }
	  if (st.rc != CMPI_RC_OK) break;
	}
	CWS_End_Enum(enumhdl);
      }
    }
    
    if (CMClassPathIsA(_broker,cop,FILECLASS,NULL) ||
	CMClassPathIsA(_broker,cop,DIRECTORYCLASS,NULL)) {
      /* we can always return the parent */
      data = CMGetKey(cop,"Name",NULL);
      if (CWS_Get_File(dirname(CMGetCharPtr(data.value.string)),&filebuf)) {
	if (names) {
	  if (associators) {
	    st = returnAssocName(rslt,cop,DIRECTORYCLASS,&filebuf,assocClass,resultClass);
	  } else {
	    st = returnRefName(rslt,cop,DIRECTORYCLASS,&filebuf,assocClass);
	  }
	}
      }
      
    }
    
    else {
      fprintf(stderr,"--- unsupported class %s\n",CMGetCharPtr(clsname));
    }
    CMReturnDone(rslt);
  } /* if (clsname) */    
  
  return st;
}

/* ------------------------------------------------------------------ *
 * Instance MI Cleanup 
 * ------------------------------------------------------------------ */

CMPIStatus CWS_DirectoryContainsFileCleanup( CMPIInstanceMI * mi, 
				 const CMPIContext * ctx,
             CMPIBoolean terminating) 
{
  CMReturn(CMPI_RC_OK);
}

/* ------------------------------------------------------------------ *
 * Instance MI Functions
 * ------------------------------------------------------------------ */


CMPIStatus CWS_DirectoryContainsFileEnumInstanceNames( CMPIInstanceMI * mi, 
					   const CMPIContext * ctx, 
					   const CMPIResult * rslt, 
					   const CMPIObjectPath * ref) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileEnumInstances( CMPIInstanceMI * mi, 
				       const CMPIContext * ctx, 
				       const CMPIResult * rslt, 
				       const CMPIObjectPath * ref, 
				       const char ** properties) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}


CMPIStatus CWS_DirectoryContainsFileGetInstance( CMPIInstanceMI * mi, 
				     const CMPIContext * ctx, 
				     const CMPIResult * rslt, 
				     const CMPIObjectPath * cop, 
				     const char ** properties) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileCreateInstance( CMPIInstanceMI * mi, 
					const CMPIContext * ctx, 
					const CMPIResult * rslt, 
					const CMPIObjectPath * cop, 
					const CMPIInstance * ci) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileModifyInstance( CMPIInstanceMI * mi, 
				     const CMPIContext * ctx, 
				     const CMPIResult * rslt, 
				     const CMPIObjectPath * cop,
				     const CMPIInstance * ci, 
				     const char **properties) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileDeleteInstance( CMPIInstanceMI * mi, 
					const CMPIContext * ctx, 
					const CMPIResult * rslt, 
					const CMPIObjectPath * cop) 
{ 
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileExecQuery( CMPIInstanceMI * mi, 
				   const CMPIContext * ctx, 
				   const CMPIResult * rslt, 
				   const CMPIObjectPath * cop, 
				   const char * lang, 
				   const char * query) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

/* ------------------------------------------------------------------ *
 * Association MI Cleanup 
 * ------------------------------------------------------------------ */

CMPIStatus CWS_DirectoryContainsFileAssociationCleanup( CMPIAssociationMI * mi,
					      const CMPIContext * ctx,
                     CMPIBoolean terminating) 
{
  CMReturn(CMPI_RC_OK);
}

/* ------------------------------------------------------------------ *
 * Association MI Functions
 * ------------------------------------------------------------------ */

CMPIStatus CWS_DirectoryContainsFileAssociators( CMPIAssociationMI * mi,
				       const CMPIContext * ctx,
				       const CMPIResult * rslt,
				       const CMPIObjectPath * cop,
				       const char * assocClass,
				       const char * resultClass,
				       const char * role,
				       const char * resultRole,
				       const char ** propertyList ) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_DirectoryContainsFileAssociatorNames( CMPIAssociationMI * mi,
					   const CMPIContext * ctx,
					   const CMPIResult * rslt,
					   const CMPIObjectPath * cop,
					   const char * assocClass,
					   const char * resultClass,
					   const char * role,
					   const char * resultRole ) 
{
  return associatorHelper(rslt,cop,1,1,assocClass,resultClass);
}

CMPIStatus CWS_DirectoryContainsFileReferences( CMPIAssociationMI * mi,
				      const CMPIContext * ctx,
				      const CMPIResult * rslt,
				      const CMPIObjectPath * cop,
				      const char * assocClass,
				      const char * role,
				      const char ** propertyList ) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}


CMPIStatus CWS_DirectoryContainsFileReferenceNames( CMPIAssociationMI * mi,
					  const CMPIContext * ctx,
					  const CMPIResult * rslt,
					  const CMPIObjectPath * cop,
					  const char * assocClass,
					  const char * role) 
{
  return associatorHelper(rslt,cop,0,1,assocClass,NULL);
}

/* ------------------------------------------------------------------ *
 * Instance MI Factory
 * 
 * NOTE: This is an example using the convenience macros. This is OK 
 *       as long as the MI has no special requirements, i.e. to store
 *       data between calls.
 * ------------------------------------------------------------------ */

CMInstanceMIStub( CWS_DirectoryContainsFile,
		  CWS_DirectoryContainsFileProvider,
		  _broker,
		  CMNoHook);

/* ------------------------------------------------------------------ *
 * Association MI Factory
 * ------------------------------------------------------------------ */

CMAssociationMIStub( CWS_DirectoryContainsFile,
		     CWS_DirectoryContainsFileProvider,
		     _broker,
		     CMNoHook);
