/*
 * $Id: CWS_FileUtils.c,v 1.4 2010/08/05 22:41:52 tyreld Exp $
 *
 * (C) Copyright IBM Corp. 2003, 2005, 2010
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Eclipse Public License from
 * http://www.opensource.org/licenses/eclipse-1.0.php
 *
 * Author:      Viktor Mihajlovski <mihajlov@de.ibm.com>
 *
 * Description: Utility functions for CMPI Workshop
 * 
 */

#include "CWS_FileUtils.h"
#include "util/cwsutil.h"
#include <cmpidt.h>
#include <cmpift.h>
#include <cmpimacs.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>


static char cscn[] = "CIM_UnitaryComputerSystem";
static char fscn[] = "CIM_FileSystem";
static char csn[500] = "";
static char fsn[] = CWS_FILEROOT;

char * CWS_CSCreationClassName()
{
  return cscn;
}

char * CWS_CSName()
{
  if (*csn == 0)
    gethostname(csn,sizeof(csn));
  return csn;
}

char * CWS_FSCreationClassName()
{
  return fscn;
}

char * CWS_FSName()
{
  return fsn;
}


CMPIObjectPath *CWS_makePath(const CMPIBroker *broker, const char * classname, 
			 const char * namespace, CWS_FILE *cwsf)
{
  CMPIObjectPath *op;
  op = CMNewObjectPath(broker, 
		       (char*)namespace,
		       (char*)classname,
		       NULL);
  if (!CMIsNullObject(op)) {
    CMAddKey(op,"CSCreationClassName",CWS_CSCreationClassName(),CMPI_chars); 
    CMAddKey(op,"CSName",CWS_CSName(),CMPI_chars); 
    CMAddKey(op,"FSCreationClassName",CWS_FSCreationClassName(),CMPI_chars); 
    CMAddKey(op,"FSName",CWS_FSName(),CMPI_chars); 
    CMAddKey(op,"CreationClassName",classname,CMPI_chars); 
    CMAddKey(op,"Name",cwsf->cws_name,CMPI_chars); 
  }
  return op;
}

CMPIInstance   *CWS_makeInstance(const CMPIBroker *broker, const char * classname, 
			     const char * namespace, CWS_FILE *cwsf, 
			     const char ** filter)
{
  CMPIInstance   *in = NULL;
  CMPIValue       val;
  CMPIObjectPath *op = CMNewObjectPath(broker, 
				       (char*)namespace,
				       (char*)classname,
				       NULL);
  
  if (!CMIsNullObject(op)) {
    in = CMNewInstance(broker,op,NULL);
    if (!CMIsNullObject(in)) {
      if (filter) {
	static const char * keys[] = {
	  "CSCreationClassName",
	  "CSName",
	  "FSCreationClassName",
	  "FSName",
	  "CreationClassName",
	  "Name"
	};
	CMSetPropertyFilter(in,filter,keys);
      }
      CMSetProperty(in,"CSCreationClassName",CWS_CSCreationClassName(),CMPI_chars);
      CMSetProperty(in,"CSName",CWS_CSName(),CMPI_chars);
      CMSetProperty(in,"FSCreationClassName",CWS_FSCreationClassName(),CMPI_chars);
      CMSetProperty(in,"FSName",CWS_FSName(),CMPI_chars);
      CMSetProperty(in,"CreationClassName",classname,CMPI_chars);
      CMSetProperty(in,"Name",cwsf->cws_name,CMPI_chars); 
      CMSetProperty(in,"FileSize",(CMPIValue*)&cwsf->cws_size,CMPI_uint64);
      val.uint64 = cwsf->cws_ctime;
      val.dateTime = CMNewDateTimeFromBinary(broker,val.uint64*1000000,0,NULL);
      CMSetProperty(in,"CreationDate",&val,CMPI_dateTime);
      val.uint64 = cwsf->cws_mtime;
      val.dateTime = CMNewDateTimeFromBinary(broker,val.uint64*1000000,0,NULL);
      CMSetProperty(in,"LastModified",&val,CMPI_dateTime);
      val.uint64 = cwsf->cws_atime;
      val.dateTime = CMNewDateTimeFromBinary(broker,val.uint64*1000000,0,NULL);
      CMSetProperty(in,"LastAccessed",&val,CMPI_dateTime);
      val.uint64=0L;
      val.boolean=(cwsf->cws_mode & 0400) != 0;
      CMSetProperty(in,"Readable",&val,CMPI_boolean);
      val.boolean=(cwsf->cws_mode & 0200) != 0;
      CMSetProperty(in,"Writeable",&val,CMPI_boolean);
      val.boolean=(cwsf->cws_mode & 0100) != 0;
      CMSetProperty(in,"Executable",&val,CMPI_boolean);
    }
  }
  return in;
}

int CWS_makeFileBuf(const CMPIInstance *instance, CWS_FILE *cwsf)
{
  CMPIData dt; 
  if (instance && cwsf) {
    dt=CMGetProperty(instance,"Name",NULL);
    strcpy(cwsf->cws_name,CMGetCharPtr(dt.value.string));
    dt=CMGetProperty(instance,"FileSize",NULL);
    cwsf->cws_size=dt.value.uint64;
    //dt=CMGetProperty(instance,"CreationDate",NULL);
    //cwsf->cws_ctime=CMGetBinaryFormat(dt.value.dateTime,NULL);
    //dt=CMGetProperty(instance,"LastModified",NULL);
    //cwsf->cws_mtime=CMGetBinaryFormat(dt.value.dateTime,NULL);
    //dt=CMGetProperty(instance,"LastAccessed",NULL);
    //cwsf->cws_atime=CMGetBinaryFormat(dt.value.dateTime,NULL);
    dt=CMGetProperty(instance,"Readable",NULL);
    cwsf->cws_mode=dt.value.boolean ? 0400 : 0;
    dt=CMGetProperty(instance,"Writeable",NULL);
    cwsf->cws_mode+=(dt.value.boolean ? 0200 : 0);
    dt=CMGetProperty(instance,"Executable",NULL);
    cwsf->cws_mode+=(dt.value.boolean ? 0100 : 0);
    return 1;
  }
  return 0;
}

char **CWS_projection2Filter(CMPIArray *ar)
{
  CMPIStatus rc = {CMPI_RC_OK,NULL};
  char     **filter;
  int        i;
  int        count=0;
  
  if (ar)
    count = CMGetArrayCount(ar,NULL);
  if (count==0) return NULL;
  filter = calloc(count+1,sizeof(char*));
  for (i=0; i<count;i++) {
    CMPIData dt = CMGetArrayElementAt(ar,i,&rc);
    if (rc.rc != CMPI_RC_OK) {
      CWS_freeFilter(filter);
      filter=NULL;
      break;
    }
    filter[i] = (char *) CMGetCharsPtr(dt.value.string,NULL);
  }
  return filter;
}

void CWS_freeFilter(char **filter)
{
  if (filter) free(filter);
} 
