/*
 * CWS_PlainFile.c
 *
 * (C) Copyright IBM Corp. 2003, 2005, 2010
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Eclipse Public License from
 * http://www.opensource.org/licenses/eclipse-1.0.php
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>

 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: CMPI Workshop Plain File Instance Provider.
 * 
*/


#include "CWS_FileUtils.h"
#include "util/cwsutil.h"
#include <cmpidt.h>
#include <cmpift.h>
#include <cmpimacs.h>
#include <stdio.h>

#define LOCALCLASSNAME "CWS_PlainFile"

static const CMPIBroker * _broker;

/* ------------------------------------------------------------------ *
 * Instance MI Cleanup 
 * ------------------------------------------------------------------ */

CMPIStatus CWS_PlainFileCleanup( CMPIInstanceMI * mi, 
				 const CMPIContext * ctx,
             CMPIBoolean terminating) 
{
  CMReturn(CMPI_RC_OK);
}

/* ------------------------------------------------------------------ *
 * Instance MI Functions
 * ------------------------------------------------------------------ */


CMPIStatus CWS_PlainFileEnumInstanceNames( CMPIInstanceMI * mi, 
					   const CMPIContext * ctx, 
					   const CMPIResult * rslt, 
					   const CMPIObjectPath * ref) 
{
  CMPIObjectPath *op;
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  void           *enumhdl;
  CWS_FILE        filebuf;

  fprintf(stderr,"--- CWS_PlainFileEnumInstanceNames() \n");
  
  enumhdl = CWS_Begin_Enum(CWS_FILEROOT,CWS_TYPE_PLAIN,1);
  
  if (enumhdl == NULL) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not begin file enumeration");
    return st;
  } else {
    while (CWS_Next_Enum(enumhdl,&filebuf)) {
      /* build object path from file buffer */
      op = CWS_makePath(_broker, 
		    LOCALCLASSNAME,
		    CMGetCharPtr(CMGetNameSpace(ref,NULL)), 
		    &filebuf);
      if (CMIsNullObject(op)) {
	CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			     "Could not construct object path");
	break;
      }
      CMReturnObjectPath(rslt,op);
    }
    CMReturnDone(rslt);
    CWS_End_Enum(enumhdl);
  }
 
  return st;
}

CMPIStatus CWS_PlainFileEnumInstances( CMPIInstanceMI * mi, 
				       const CMPIContext * ctx, 
				       const CMPIResult * rslt, 
				       const CMPIObjectPath * ref, 
				       const char ** properties) 
{
  CMPIInstance   *in; 
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  void           *enumhdl;
  CWS_FILE        filebuf;
  
  fprintf(stderr,"--- CWS_PlainFileEnumInstances() \n");
  
  enumhdl = CWS_Begin_Enum(CWS_FILEROOT,CWS_TYPE_PLAIN,1);
  
  if (enumhdl == NULL) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not begin file enumeration");
    return st;
  } else {
    while (CWS_Next_Enum(enumhdl,&filebuf)) {
      /* build instance from file buffer */
      in = CWS_makeInstance(_broker, 
			LOCALCLASSNAME,
			CMGetCharPtr(CMGetNameSpace(ref,NULL)), 
			&filebuf,NULL);
      if (CMIsNullObject(in)) {
	CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			     "Could not construct instance");
	break;
      }
      CMReturnInstance(rslt,in);
    }
    CMReturnDone(rslt);
    CWS_End_Enum(enumhdl);
  }
 
  return st;
}


CMPIStatus CWS_PlainFileGetInstance( CMPIInstanceMI * mi, 
				     const CMPIContext * ctx, 
				     const CMPIResult * rslt, 
				     const CMPIObjectPath * cop, 
				     const char ** properties) 
{
  CMPIInstance *in = NULL;
  CMPIStatus    st = {CMPI_RC_OK,NULL};
  CMPIData      nd = CMGetKey(cop,"Name",&st);
  CWS_FILE      filebuf;

  fprintf(stderr,"--- CWS_PlainFileGetInstance() \n");
  
  if (st.rc == CMPI_RC_OK &&
      nd.type == CMPI_string &&
      CWS_Get_File(CMGetCharPtr(nd.value.string),&filebuf))
    in = CWS_makeInstance(_broker, 
		      LOCALCLASSNAME,
		      CMGetCharPtr(CMGetNameSpace(cop,NULL)), 
		      &filebuf,NULL);
  
  if (CMIsNullObject(in)) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not find or construct instance");
  } else {
    CMReturnInstance(rslt,in);    
    CMReturnDone(rslt);
  }
  
  return st;
}

CMPIStatus CWS_PlainFileCreateInstance( CMPIInstanceMI * mi, 
					const CMPIContext * ctx, 
					const CMPIResult * rslt, 
					const CMPIObjectPath * cop, 
					const CMPIInstance * ci) 
{
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_PlainFileModifyInstance( CMPIInstanceMI * mi, 
				     const CMPIContext * ctx, 
				     const CMPIResult * rslt, 
				     const CMPIObjectPath * cop,
				     const CMPIInstance * ci, 
				     const char **properties) 
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CWS_FILE   filebuf;
  
  fprintf(stderr,"--- CWS_PlainFileModifyInstance() \n");
  
  if (!CWS_makeFileBuf(ci,&filebuf) || !CWS_Update_File(&filebuf))
    CMSetStatusWithChars(_broker,&st,CMPI_RC_ERR_FAILED,
			 "Could not update instance");
  
  return st;
}

CMPIStatus CWS_PlainFileDeleteInstance( CMPIInstanceMI * mi, 
					const CMPIContext * ctx, 
					const CMPIResult * rslt, 
					const CMPIObjectPath * cop) 
{ 
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus CWS_PlainFileExecQuery( CMPIInstanceMI * mi, 
				   const CMPIContext * ctx, 
				   const CMPIResult * rslt, 
				   const CMPIObjectPath * cop, 
				   const char * query, 
				   const char * lang) 
{
  CMPIInstance   *in; 
  CMPIStatus      st = {CMPI_RC_OK,NULL};
  void           *enumhdl;
  CWS_FILE        filebuf;
  CMPIArray      *ar;
  CMPISelectExp  *se;
  char           **filter;
  
  fprintf(stderr,"--- CWS_PlainFileExecQuery() \n");
  
  se = CMNewSelectExp(_broker,query,lang,&ar,&st);
  
  if (se==NULL || st.rc != CMPI_RC_OK) {
    fprintf(stderr,"--- CWS_PlainFileExecQuery() error in newselectexp:"
	    "se=%p, rc=%d, msg=\"%s\"\n",se,st.rc,CMGetCharPtr(st.msg));
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not create select expression");
    return st;
  }
  
  enumhdl = CWS_Begin_Enum(CWS_FILEROOT,CWS_TYPE_PLAIN,1);
  
  if (enumhdl == NULL) {
    CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			 "Could not begin file enumeration");
    fprintf(stderr,"--- CWS_PlainFileExecQuery() error in beginenum \n");
    return st;
  } else {
    filter = CWS_projection2Filter(ar);
        while (CWS_Next_Enum(enumhdl,&filebuf)) {
      /* build instance from file buffer */
      in = CWS_makeInstance(_broker, 
			LOCALCLASSNAME,
			CMGetCharPtr(CMGetNameSpace(cop,NULL)), 
			&filebuf,(const char **)filter);
      if (CMIsNullObject(in)) {
	CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			     "Could not construct instance");
	break;
      }
      if (!CMEvaluateSelExp(se,in,&st)) {
	if (st.rc != CMPI_RC_OK) {
	  fprintf(stderr,"--- CWS_PlainFileExecQuery() error in evaluate:"
		  "rc=%d, msg=\"%s\"\n",st.rc,CMGetCharPtr(st.msg));
	  break;
	}
      } else {
	CMReturnInstance(rslt,in);
      }
    }
    CWS_freeFilter(filter);
    CMReturnDone(rslt);
    CWS_End_Enum(enumhdl);
  }
 
  return st;
}

/* ------------------------------------------------------------------ *
 * Method MI Cleanup 
 * ------------------------------------------------------------------ */

CMPIStatus CWS_PlainFileMethodCleanup( CMPIMethodMI * mi, 
				       const CMPIContext * ctx,
                   CMPIBoolean terminating) 
{
  CMReturn(CMPI_RC_OK);
}

/* ------------------------------------------------------------------ *
 * Method MI Functions
 * ------------------------------------------------------------------ */

CMPIStatus CWS_PlainFileInvokeMethod( CMPIMethodMI * mi, 
				      const CMPIContext * ctx, 
				      const CMPIResult * rslt,
				      const CMPIObjectPath * cop,
				      const char * method,
				      const CMPIArgs * in,
				      CMPIArgs * out)
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIData   dt;
  char typebuf[1000];
  
  fprintf(stderr,"--- CWS_PlainFileInvokeMethod() \n");
  
  dt=CMGetKey(cop,"Name",&st);
  if (st.rc != CMPI_RC_OK) {
    CMSetStatusWithChars(_broker,&st,CMPI_RC_ERR_FAILED,
			 "Could not get instance name");
  } else if (CWS_Get_FileType(CMGetCharPtr(dt.value.string),typebuf,
			      sizeof(typebuf))) {
    CMSetStatusWithChars(_broker,&st,CMPI_RC_ERR_FAILED,
			 "Could not get type");
  } else {
    CMReturnData(rslt,typebuf,CMPI_chars);
    CMReturnDone(rslt);
  }
  
  return st;
}

/* ------------------------------------------------------------------ *
 * Instance MI Factory
 * 
 * NOTE: This is an example using the convenience macros. This is OK 
 *       as long as the MI has no special requirements, i.e. to store
 *       data between calls.
 * ------------------------------------------------------------------ */

CMInstanceMIStub( CWS_PlainFile,
		  CWS_PlainFileProvider,
		  _broker,
		  CMNoHook);

/* ------------------------------------------------------------------ *
 * Method MI Factory
 * 
 * ------------------------------------------------------------------ */

CMMethodMIStub( CWS_PlainFile,
		CWS_PlainFileProvider,
		_broker,
		CMNoHook);
